! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

module ero_general

  use iso_fortran_env
  implicit none

! Bit position constants:
  integer, parameter :: BIT_MPE_OWNER              =  0	! set by task EVATT
  integer, parameter :: BIT_IKI_OWNER              =  1	! set by task EVATT

  integer, parameter :: BIT_TRAILING_EVENT         =  4	! set by task PATTERN
  integer, parameter :: BIT_NEXT_TO_BORDER         =  5	! set by task EVPREP

  integer, parameter :: BIT_NEXT_TO_ONBOARD_BADPIX =  8	! set by task FTFINDHOTPIX
  integer, parameter :: BIT_NEXT_TO_BRIGHT_PIX     =  9	! set by task FTFINDHOTPIX
  integer, parameter :: BIT_NEXT_TO_DEAD_PIX       = 10	! set by task FTFINDHOTPIX
  integer, parameter :: BIT_NEXT_TO_FLICKERING     = 11	! set by task FTFINDHOTPIX 
  integer, parameter :: BIT_ON_FLICKERING          = 12	! set by task FTFINDHOTPIX
  integer, parameter :: BIT_ON_BADPIX              = 13	! set by task FTFINDHOTPIX
  integer, parameter :: BIT_ON_DEADPIX             = 14	! set by task FTFINDHOTPIX
  integer, parameter :: BIT_OUT_OF_FOV             = 15	! set by task EVPREP
  integer, parameter :: BIT_OUTSIDE_QUALGTI        = 16	! set by task QUALGTI
  integer, parameter :: BIT_OUTSIDE_GTI            = 17	! set by task EVPREP
  integer, parameter :: BIT_PRECEDING_MIP          = 18	! set by task EVPREP
  integer, parameter :: BIT_MIP_ASSOCIATED         = 19	! set by task EXPOSURE?

  integer, parameter :: BIT_PHA_QUALITY_1          = 24	! set by task EVPREP
  integer, parameter :: BIT_PHA_QUALITY_2          = 25	! set by task EVPREP
  integer, parameter :: BIT_PHA_QUALITY_3          = 26	! set by task EVPREP

  integer, parameter :: BIT_CORRUPT_EVENT          = 30	! set by task EVPREP
  integer, parameter :: BIT_CORRUPT_FRAME          = 31	! set by task EVPREP

! Event flag bit masks
  integer(kind=int32), parameter :: zero_int32 = 0

! Owner flags:
  integer(kind=int32), parameter :: FLAG_MPE_OWNER              = ibset(zero_int32,BIT_MPE_OWNER)  ! 0x1
  ! 0: MPE is owner
  integer(kind=int32), parameter :: FLAG_IKI_OWNER              = ibset(zero_int32,BIT_IKI_OWNER)  ! 0x2
  ! 0: IKI is owner

! Information flags:
  integer(kind=int32), parameter :: FLAG_TRAILING_EVENT         = ibset(zero_int32,BIT_TRAILING_EVENT)  ! 0x10
  integer(kind=int32), parameter :: FLAG_NEXT_TO_BORDER         = ibset(zero_int32,BIT_NEXT_TO_BORDER)  ! 0x20

  integer(kind=int32), parameter :: FLAG_NEXT_TO_ONBOARD_BADPIX = ibset(zero_int32,BIT_NEXT_TO_ONBOARD_BADPIX)  ! 0x100
  integer(kind=int32), parameter :: FLAG_NEXT_TO_BRIGHT_PIX     = ibset(zero_int32,BIT_NEXT_TO_BRIGHT_PIX)  ! 0x200
  integer(kind=int32), parameter :: FLAG_NEXT_TO_DEAD_PIX       = ibset(zero_int32,BIT_NEXT_TO_DEAD_PIX)  ! 0x400
  integer(kind=int32), parameter :: FLAG_NEXT_TO_FLICKERING     = ibset(zero_int32,BIT_NEXT_TO_FLICKERING)  ! 0x800

! Rejection flags:
  integer(kind=int32), parameter :: FLAG_ON_FLICKERING          = ibset(zero_int32,BIT_ON_FLICKERING)  ! 0x1000 
  integer(kind=int32), parameter :: FLAG_ON_BADPIX              = ibset(zero_int32,BIT_ON_BADPIX)  ! 0x2000
  integer(kind=int32), parameter :: FLAG_ON_DEADPIX             = ibset(zero_int32,BIT_ON_DEADPIX)  ! 0x4000
  integer(kind=int32), parameter :: FLAG_OUT_OF_FOV             = ibset(zero_int32,BIT_OUT_OF_FOV)  ! 0x8000

  integer(kind=int32), parameter :: FLAG_OUTSIDE_QUALGTI        = ibset(zero_int32,BIT_OUTSIDE_QUALGTI)  ! 0x10000
  integer(kind=int32), parameter :: FLAG_OUTSIDE_GTI            = ibset(zero_int32,BIT_OUTSIDE_GTI)  ! 0x20000
  integer(kind=int32), parameter :: FLAG_PRECEDING_MIP          = ibset(zero_int32,BIT_PRECEDING_MIP)  ! 0x40000
  integer(kind=int32), parameter :: FLAG_MIP_ASSOCIATED         = ibset(zero_int32,BIT_MIP_ASSOCIATED)  ! 0x80000

! Quality flags:
  integer(kind=int32), parameter :: FLAG_PHA_QUALITY_1          = ibset(zero_int32,BIT_PHA_QUALITY_1)  ! 0x1000000
  integer(kind=int32), parameter :: FLAG_PHA_QUALITY_2          = ibset(zero_int32,BIT_PHA_QUALITY_2)  ! 0x2000000
  integer(kind=int32), parameter :: FLAG_PHA_QUALITY_3          = ibset(zero_int32,BIT_PHA_QUALITY_3)  ! 0x4000000

! Corrupted data:
  integer(kind=int32), parameter :: FLAG_CORRUPT_EVENT          = ibset(zero_int32,BIT_CORRUPT_EVENT)  ! 0x40000000
  integer(kind=int32), parameter :: FLAG_CORRUPT_FRAME          = ibset(zero_int32,BIT_CORRUPT_FRAME)  ! 0x80000000
end module ero_general

